@@ -28,16 +28,17 @@
    NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
    */
-#include <sys/poll.h>
+#include "client.h"
 #include <sys/socket.h>
 #include <fcntl.h>
 #include <errno.h>
 #include <limits.h>
+#include <poll.h>
 #include <stdlib.h>
 #include <string.h>
 #include "log.h"
+#include "memory.h"
 #include "list.h"
-#include "client.h"
 #include "ssl.h"
 #include "messages.h"
 #include "messagehandler.h"
@@ -66,12 +67,12 @@ bool_t bPreferAlpha;
 extern int* udpsocks;
 extern bool_t hasv4;
 
-void Client_init()
+void Client_init(void)
 {
 	maxBandwidth = getIntConf(MAX_BANDWIDTH) / 8; /* From bits/s -> bytes/s */
 }
 
-int Client_count()
+int Client_count(void)
 {
 	return clientcount;
 }
@@ -92,7 +93,7 @@ int Client_getfds(struct pollfd *pollfds)
 	return i;
 }
 
-void Client_janitor()
+void Client_janitor(void)
 {
 	struct dlist *itr, *save;
 	int bwTop = maxBandwidth + maxBandwidth / 4;
@@ -115,9 +116,7 @@ void Client_janitor()
 
 void Client_codec_add(client_t *client, int codec)
 {
-	codec_t *cd = malloc(sizeof(codec_t));
-	if (cd == NULL)
-		Log_fatal("Out of memory");
+	codec_t *cd = Memory_safeMalloc(1, sizeof(codec_t));
 	init_list_entry(&cd->node);
 	cd->codec = codec;
 	list_add_tail(&cd->node, &client->codecs);
@@ -157,9 +156,7 @@ void Client_token_add(client_t *client, char *token_string)
 
 	if (client->tokencount >= MAX_TOKENS)
 		return;
-	token = malloc(sizeof(token_t));
-	if (token == NULL)
-		Log_fatal("Out of memory");
+	token = Memory_safeMalloc(1, sizeof(token_t));
 	init_list_entry(&token->node);
 	token->token = strdup(token_string);
 	if (token->token == NULL)
@@ -227,9 +224,7 @@ void recheckCodecVersions(client_t *connectingClient)
 				}
 			}
 			if (!found) {
-				cd = malloc(sizeof(codec_t));
-				if (!cd)
-					Log_fatal("Out of memory");
+				cd = Memory_safeMalloc(1, sizeof(codec_t));
 				memset(cd, 0, sizeof(codec_t));
 				init_list_entry(&cd->node);
 				cd->codec = codec_itr->codec;
@@ -302,7 +297,7 @@ void recheckCodecVersions(client_t *connectingClient)
 	bOpus = enableOpus;
 }
 
-static int findFreeSessionId()
+static int findFreeSessionId(void)
 {
 	int id;
 	client_t *itr = NULL;
@@ -332,8 +327,7 @@ int Client_add(int fd, struct sockaddr_storage *remote)
 		return -1;
 	}
 
-	if ((newclient = calloc(1, sizeof(client_t))) == NULL)
-		Log_fatal("(%s:%s): Out of memory while allocating %d bytes.", __FILE__, __LINE__, sizeof(client_t));
+	newclient = Memory_safeCalloc(1, sizeof(client_t));
 
 	newclient->tcpfd = fd;
 	memcpy(&newclient->remote_tcp, remote, sizeof(struct sockaddr_storage));
@@ -428,7 +422,7 @@ void Client_close(client_t *client)
 	client->shutdown_wait = true;
 }
 
-void Client_disconnect_all()
+void Client_disconnect_all(void)
 {
 	struct dlist *itr, *save;
 
@@ -623,10 +617,6 @@ int Client_send_message_ver(client_t *client, message_t *msg, uint32_t version)
 
 int Client_send_message(client_t *client, message_t *msg)
 {
-	if (!client->authenticated && msg->messageType != Version) {
-		Msg_free(msg);
-		return 0;
-	}
 	if (client->txsize != 0 || !client->SSLready) {
 		/* Queue message */
 		if ((client->txQueueCount > 5 &&  msg->messageType == UDPTunnel) ||
@@ -654,10 +644,9 @@ client_t *Client_iterate(client_t **client_itr)
 {
 	client_t *c = *client_itr;
 
-	if (list_empty(&clients))
-		return NULL;
-
-	if (c == NULL) {
+	if (list_empty(&clients)) {
+		c = NULL;
+	} else if (c == NULL) {
 		c = list_get_entry(list_get_first(&clients), client_t, node);
 	} else {
 		if (list_get_next(&c->node) == &clients)
@@ -669,18 +658,22 @@ client_t *Client_iterate(client_t **client_itr)
 	return c;
 }
 
+client_t *Client_iterate_authenticated(client_t **client_itr)
+{
+	while (Client_iterate(client_itr))
+		if (IS_AUTH(*client_itr))
+			break;
+	return *client_itr;
+}
+
 void Client_textmessage(client_t *client, char *text)
 {
 	char *message;
 	uint32_t *tree_id;
 	message_t *sendmsg = NULL;
 
-	message = malloc(strlen(text) + 1);
-	if (!message)
-		Log_fatal("Out of memory");
-	tree_id = malloc(sizeof(uint32_t));
-	if (!tree_id)
-		Log_fatal("Out of memory");
+	message = Memory_safeMalloc(1, strlen(text) + 1);
+	tree_id = Memory_safeMalloc(1, sizeof(uint32_t));
 	*tree_id = 0;
 	sendmsg = Msg_create(TextMessage);
 	sendmsg->payload.textMessage->message = message;
@@ -697,7 +690,7 @@ int Client_send_message_except(client_t *client, message_t *msg)
 	int count = 0;
 
 	Msg_inc_ref(msg); /* Make sure a reference is held during the whole iteration. */
-	while (Client_iterate(&itr) != NULL) {
+	while (Client_iterate_authenticated(&itr)) {
 		if (itr != client) {
 			if (count++ > 0)
 				Msg_inc_ref(msg); /* One extra reference for each new copy */
@@ -720,7 +713,7 @@ int Client_send_message_except_ver(client_t *client, message_t *msg, uint32_t ve
 	int count = 0;
 
 	Msg_inc_ref(msg); /* Make sure a reference is held during the whole iteration. */
-	while (Client_iterate(&itr) != NULL) {
+	while (Client_iterate_authenticated(&itr)) {
 		if (itr != client) {
 			if (count++ > 0)
 				Msg_inc_ref(msg); /* One extra reference for each new copy */
@@ -906,7 +899,7 @@ int Client_voiceMsg(client_t *client, uint8_t *data, int len)
 	int offset, packetsize;
 	voicetarget_t *vt;
 
-	channel_t *ch = (channel_t *)client->channel;
+	channel_t *ch = client->channel;
 	struct dlist *itr;
 
 	if (!client->authenticated || client->mute || client->self_mute || ch->silent)
@@ -951,6 +944,21 @@ int Client_voiceMsg(client_t *client, uint8_t *data, int len)
 			c = list_get_entry(itr, client_t, chan_node);
 			Client_send_voice(client, c, buffer, pds->offset + 1, poslen);
 		}
+		if (!list_empty(&ch->channel_links)) { /* Speech to links */
+		    struct dlist *ch_itr;
+		    list_iterate(ch_itr, &ch->channel_links) {
+			channellist_t *chl;
+			channel_t *ch_link;
+			chl = list_get_entry(ch_itr, channellist_t, node);
+			ch_link = chl->chan;
+			list_iterate(itr, &ch_link->clients) {
+			    client_t *c;
+			    c = list_get_entry(itr, client_t, chan_node);
+			    Log_debug("Linked voice from %s -> %s", ch->name, ch_link->name);
+			    Client_send_voice(client, c, buffer, pds->offset + 1, poslen);
+			}
+		    }
+		}
 	} else if ((vt = Voicetarget_get_id(client, target)) != NULL) { /* Targeted whisper */
 		int i;
 		channel_t *ch;
@@ -966,7 +974,7 @@ int Client_voiceMsg(client_t *client, uint8_t *data, int len)
 				c = list_get_entry(itr, client_t, chan_node);
 				Client_send_voice(client, c, buffer, pds->offset + 1, poslen);
 			}
-			/* Channel links */
+			/* Whisper to channel links? */
 			if (vt->channels[i].linked && !list_empty(&ch->channel_links)) {
 				struct dlist *ch_itr;
 				list_iterate(ch_itr, &ch->channel_links) {
@@ -977,12 +985,12 @@ int Client_voiceMsg(client_t *client, uint8_t *data, int len)
 					list_iterate(itr, &ch_link->clients) {
 						client_t *c;
 						c = list_get_entry(itr, client_t, chan_node);
-						Log_debug("Linked voice from %s -> %s", ch->name, ch_link->name);
+						Log_debug("Linked whisper from %s -> %s", ch->name, ch_link->name);
 						Client_send_voice(client, c, buffer, pds->offset + 1, poslen);
 					}
 				}
 			}
-			/* children */
+			/* Whisper to children? */
 			if (vt->channels[i].children) {
 				struct dlist chanlist, *ch_itr;
 				init_list_entry(&chanlist);
@@ -993,14 +1001,14 @@ int Client_voiceMsg(client_t *client, uint8_t *data, int len)
 					list_iterate(itr, &sub->clients) {
 						client_t *c;
 						c = list_get_entry(itr, client_t, chan_node);
-						Log_debug("Child voice from %s -> %s", ch->name, sub->name);
+						Log_debug("Whisper to child from %s -> %s", ch->name, sub->name);
 						Client_send_voice(client, c, buffer, pds->offset + 1, poslen);
 					}
 				}
 				Chan_freeTreeList(&chanlist);
 			}
 		}
-		/* Sessions */
+		/* Whisper to sessions (users)? */
 		for (i = 0; i < TARGET_MAX_SESSIONS && vt->sessions[i] != -1; i++) {
 			client_t *c;
 			buffer[0] = (uint8_t) (type | 2);
@@ -1029,14 +1037,11 @@ static int Client_send_udp(client_t *client, uint8_t *data, int len)
 	if (Util_clientAddressToPortUDP(client) != 0 && CryptState_isValid(&client->cryptState) &&
 		client->bUDP) {
 #if defined(__LP64__)
-		buf = mbuf = malloc(len + 4 + 16);
+		buf = mbuf = Memory_safeMalloc(1, len + 4 + 16);
 		buf += 4;
 #else
-		mbuf = buf = malloc(len + 4);
+		mbuf = buf = Memory_safeMalloc(1, len + 4);
 #endif
-		if (mbuf == NULL)
-			Log_fatal("Out of memory");
-
 		CryptState_encrypt(&client->cryptState, data, buf, len);
 
 #if defined(__NetBSD__) || defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__APPLE__)
