@@ -28,6 +28,8 @@
    NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
+#include "conf.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -35,7 +37,6 @@
 #include <libconfig.h>
 
 #include "types.h"
-#include "conf.h"
 #include "log.h"
 
 static config_t configuration;
@@ -75,7 +76,7 @@ bool_t Conf_ok(const char *conffile)
 	return rc;
 }
 
-void Conf_deinit()
+void Conf_deinit(void)
 {
 	config_destroy(&configuration);
 }
