@@ -28,14 +28,16 @@
    NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
+#include "messagehandler.h"
+
 #include <string.h>
 #include <stdlib.h>
 
 #include "log.h"
+#include "memory.h"
 #include "list.h"
 #include "client.h"
 #include "messages.h"
-#include "messagehandler.h"
 #include "crypt.h"
 #include "channel.h"
 #include "conf.h"
@@ -140,12 +142,8 @@ void Mh_handle_message(client_t *client, message_t *msg)
 			goto disconnect;
 		}
 
-		client->authenticated = true;
-
 		client_itr = NULL;
-		while (Client_iterate(&client_itr) != NULL) {
-			if (!IS_AUTH(client_itr))
-				continue;
+		while (Client_iterate_authenticated(&client_itr)) {
 			if (client_itr->username && strncmp(client_itr->username, msg->payload.authenticate->username, MAX_USERNAME) == 0) {
 				char buf[64];
 				sprintf(buf, "Username already in use");
@@ -214,6 +212,8 @@ void Mh_handle_message(client_t *client, message_t *msg)
 		/* Channel stuff */
 		Chan_userJoin(defaultChan, client); /* Join default channel */
 
+		client->authenticated = true;
+
 		/* Codec version */
 		Log_debug("Client %d has %d CELT codecs", client->sessionId,
 				  msg->payload.authenticate->n_celt_versions);
@@ -284,7 +284,9 @@ void Mh_handle_message(client_t *client, message_t *msg)
 				sendmsg->payload.channelState->channel_id = ch_itr->id;
 				sendmsg->payload.channelState->n_links = ch_itr->linkcount;
 
-				links = (uint32_t *)malloc(ch_itr->linkcount * sizeof(uint32_t));
+				links = (uint32_t*)Memory_safeMalloc(
+					ch_itr->linkcount,
+					sizeof(uint32_t));
 				list_iterate(itr, &ch_itr->channel_links) { /* Iterate links */
 					channellist_t *chl;
 					channel_t *ch;
@@ -303,7 +305,7 @@ void Mh_handle_message(client_t *client, message_t *msg)
 		sendmsg->payload.userState->session = client->sessionId;
 		sendmsg->payload.userState->name = strdup(client->username);
 		sendmsg->payload.userState->has_channel_id = true;
-		sendmsg->payload.userState->channel_id = ((channel_t *)client->channel)->id;
+		sendmsg->payload.userState->channel_id = client->channel->id;
 
 		if (defaultChan->silent) {
 			sendmsg->payload.userState->has_suppress = true;
@@ -313,19 +315,17 @@ void Mh_handle_message(client_t *client, message_t *msg)
 		Client_send_message_except(client, sendmsg);
 
 		client_itr = NULL;
-		while (Client_iterate(&client_itr) != NULL) {
-			if (!IS_AUTH(client_itr))
-				continue;
+		while (Client_iterate_authenticated(&client_itr)) {
 			sendmsg = Msg_create(UserState);
 			sendmsg->payload.userState->has_session = true;
 			sendmsg->payload.userState->session = client_itr->sessionId;
 			sendmsg->payload.userState->name = strdup(client_itr->username);
 			sendmsg->payload.userState->has_channel_id = true;
-			sendmsg->payload.userState->channel_id = ((channel_t *)client_itr->channel)->id;
-			sendmsg->payload.userState->has_suppress = ((channel_t *)client_itr->channel)->silent;
-			sendmsg->payload.userState->suppress = ((channel_t *)client_itr->channel)->silent;
+			sendmsg->payload.userState->channel_id = client_itr->channel->id;
+			sendmsg->payload.userState->has_suppress = client_itr->channel->silent;
+			sendmsg->payload.userState->suppress = client_itr->channel->silent;
 
-			client_itr->isSuppressed = ((channel_t *)client_itr->channel)->silent;
+			client_itr->isSuppressed = client_itr->channel->silent;
 
 			if (client_itr->self_deaf) {
 				sendmsg->payload.userState->has_self_deaf = true;
@@ -347,6 +347,12 @@ void Mh_handle_message(client_t *client, message_t *msg)
 				sendmsg->payload.userState->has_recording = true;
 				sendmsg->payload.userState->recording = true;
 			}
+			if (client_itr->priority_speaker) {
+				sendmsg->payload.userState->
+					has_priority_speaker = true;
+				sendmsg->payload.userState->
+					priority_speaker = true;
+			}
 			Client_send_message(client, sendmsg);
 		}
 
@@ -418,7 +424,7 @@ void Mh_handle_message(client_t *client, message_t *msg)
 		if (!msg->payload.cryptSetup->has_client_nonce) {
 			sendmsg = Msg_create(CryptSetup);
 			sendmsg->payload.cryptSetup->has_server_nonce = true;
-			sendmsg->payload.cryptSetup->server_nonce.data = client->cryptState.decrypt_iv;
+			sendmsg->payload.cryptSetup->server_nonce.data = client->cryptState.encrypt_iv;
 			sendmsg->payload.cryptSetup->server_nonce.len = AES_BLOCK_SIZE;
 			Client_send_message(client, sendmsg);
 		} else {
@@ -446,7 +452,7 @@ void Mh_handle_message(client_t *client, message_t *msg)
 		}
 
 		if (msg->payload.userState->has_user_id || msg->payload.userState->has_suppress ||
-		    msg->payload.userState->has_priority_speaker || msg->payload.userState->has_texture) {
+		    msg->payload.userState->has_texture) {
 			sendPermissionDenied(client, "Not supported by uMurmur");
 			break;
 		}
@@ -459,6 +465,14 @@ void Mh_handle_message(client_t *client, message_t *msg)
 		msg->payload.userState->has_actor = true;
 		msg->payload.userState->actor = client->sessionId;
 
+		/* Quod licet Iovi, non licet bovi */
+		if (!client->isAdmin && (msg->payload.userState->has_deaf ||
+		    msg->payload.userState->has_mute ||
+		    msg->payload.userState->has_priority_speaker)) {
+			sendPermissionDenied(client, "Permission denied");
+			break;
+		}
+
 		if (msg->payload.userState->has_deaf) {
 			target->deaf = msg->payload.userState->deaf;
 			if (target->deaf) {
@@ -474,6 +488,10 @@ void Mh_handle_message(client_t *client, message_t *msg)
 				target->deaf = false;
 			}
 		}
+		if (msg->payload.userState->has_priority_speaker) {
+			target->priority_speaker =
+				msg->payload.userState->priority_speaker;
+		}
 		if (msg->payload.userState->has_self_deaf) {
 			client->self_deaf = msg->payload.userState->self_deaf;
 			if (client->self_deaf) {
@@ -495,12 +513,8 @@ void Mh_handle_message(client_t *client, message_t *msg)
 			char *message;
 			uint32_t *tree_id;
 
-			message = malloc(strlen(client->username) + 32);
-			if (!message)
-				Log_fatal("Out of memory");
-			tree_id = malloc(sizeof(uint32_t));
-			if (!tree_id)
-				Log_fatal("Out of memory");
+			message = Memory_safeMalloc(1, strlen(client->username) + 32);
+			tree_id = Memory_safeMalloc(1, sizeof(uint32_t));
 			*tree_id = 0;
 			sendmsg = Msg_create(TextMessage);
 			sendmsg->payload.textMessage->message = message;
@@ -515,23 +529,21 @@ void Mh_handle_message(client_t *client, message_t *msg)
 		}
 		if (msg->payload.userState->has_channel_id) {
 			int leave_id;
+			channel_t *chan =
+				Chan_fromId(msg->payload.userState->channel_id);
 
-			channelJoinResult_t result = Chan_userJoin_id_test(msg->payload.userState->channel_id, target);
-
-			if (result.CHJOIN_NOENTER || result.CHJOIN_NOTFOUND)
+			if (!chan || chan->noenter)
 				break;
 
-			if (result.CHJOIN_WRONGPW) {
-				if (target == client && !client->isAdmin) {
-					sendPermissionDenied(client, "Wrong channel password");
-					break;
-				}
-				/* Tricky one: if user hasn't the password, but is moved to the channel by admin then let
-				 * the user in. Also let admin user in regardless of channel password.
-				 * Take no action on other errors.
-				 */
-				else if (!client->isAdmin)
-					break;
+			/* Tricky one: if user hasn't the password, but is moved
+			 * to the channel by admin then let the user in. Also
+			 * let admin user in regardless of channel password.
+			 */
+			if (!client->isAdmin && chan->password &&
+			    !Client_token_match(target, chan->password)) {
+				sendPermissionDenied(client,
+				                     "Wrong channel password");
+				break;
 			}
 
 			leave_id = Chan_userJoin_id(msg->payload.userState->channel_id, target);
@@ -541,7 +553,7 @@ void Mh_handle_message(client_t *client, message_t *msg)
 				sendmsg->payload.channelRemove->channel_id = leave_id;
 			}
 
-			if (result.CHJOIN_SILENT) {
+			if (chan->silent) {
 				if (!target->isSuppressed) {
 				msg->payload.userState->has_suppress = true;
 				msg->payload.userState->suppress = true;
@@ -557,9 +569,7 @@ void Mh_handle_message(client_t *client, message_t *msg)
 		if (msg->payload.userState->has_plugin_context) {
 			if (client->context)
 				free(client->context);
-			client->context = malloc(msg->payload.userState->plugin_context.len);
-			if (client->context == NULL)
-				Log_fatal("Out of memory");
+			client->context = Memory_safeMalloc(1, msg->payload.userState->plugin_context.len);
 			memcpy(client->context, msg->payload.userState->plugin_context.data,
 				   msg->payload.userState->plugin_context.len);
 
@@ -614,9 +624,7 @@ void Mh_handle_message(client_t *client, message_t *msg)
 			client_t *itr;
 			for (i = 0; i < msg->payload.textMessage->n_session; i++) {
 				itr = NULL;
-				while (Client_iterate(&itr) != NULL) {
-					if (!IS_AUTH(itr))
-						continue;
+				while (Client_iterate_authenticated(&itr)) {
 					if (itr->sessionId == msg->payload.textMessage->session[i]) {
 						if (!itr->deaf && !itr->self_deaf) {
 							Msg_inc_ref(msg);
@@ -802,9 +810,7 @@ void Mh_handle_message(client_t *client, message_t *msg)
 
 		if (!msg->payload.userStats->has_session)
 			sendPermissionDenied(client, "Not supported by uMurmur");
-		while (Client_iterate(&target) != NULL) {
-			if (!IS_AUTH(target))
-				continue;
+		while (Client_iterate_authenticated(&target)) {
 			if (target->sessionId == msg->payload.userStats->session)
 				break;
 		}
@@ -863,9 +869,8 @@ void Mh_handle_message(client_t *client, message_t *msg)
 				sendmsg->payload.userStats->version->os_version = strdup(target->os_version);
 
 			sendmsg->payload.userStats->n_celt_versions = target->codec_count;
-			sendmsg->payload.userStats->celt_versions = malloc(sizeof(int32_t) * target->codec_count);
-			if (!sendmsg->payload.userStats->celt_versions)
-				Log_fatal("Out of memory");
+			sendmsg->payload.userStats->celt_versions
+				= Memory_safeMalloc(target->codec_count, sizeof(int32_t));
 			i = 0;
 			while (Client_codec_iterate(target, &codec_itr) != NULL)
 				sendmsg->payload.userStats->celt_versions[i++] = codec_itr->codec;
@@ -874,18 +879,21 @@ void Mh_handle_message(client_t *client, message_t *msg)
 			sendmsg->payload.userStats->opus = target->bOpus;
 
 			/* Address */
-			sendmsg->payload.userStats->has_address = true;
-			sendmsg->payload.userStats->address.data = malloc(sizeof(uint8_t) * 16);
-			if (!sendmsg->payload.userStats->address.data)
-				Log_fatal("Out of memory");
-			memset(sendmsg->payload.userStats->address.data, 0, 16);
-			/* ipv4 representation as ipv6 address. Supposedly correct. */
-			memset(&sendmsg->payload.userStats->address.data[10], 0xff, 2); /* IPv4 */
-      if(target->remote_tcp.ss_family == AF_INET)
-        memcpy(&sendmsg->payload.userStats->address.data[12], &((struct sockaddr_in*)&target->remote_tcp)->sin_addr, 4);
-      else
-        memcpy(&sendmsg->payload.userStats->address.data[0], &((struct sockaddr_in6*)&target->remote_tcp)->sin6_addr, 16);
-			sendmsg->payload.userStats->address.len = 16;
+			if (getBoolConf(SHOW_ADDRESSES)) {
+				sendmsg->payload.userStats->has_address = true;
+				sendmsg->payload.userStats->address.data
+					= Memory_safeMalloc(16, sizeof(uint8_t));
+				memset(sendmsg->payload.userStats->address.data, 0, 16);
+				/* ipv4 representation as ipv6 address. Supposedly correct. */
+				memset(&sendmsg->payload.userStats->address.data[10], 0xff, 2); /* IPv4 */
+				if(target->remote_tcp.ss_family == AF_INET)
+					memcpy(&sendmsg->payload.userStats->address.data[12], &((struct sockaddr_in*)&target->remote_tcp)->sin_addr, 4);
+				else
+					memcpy(&sendmsg->payload.userStats->address.data[0], &((struct sockaddr_in6*)&target->remote_tcp)->sin6_addr, 16);
+				sendmsg->payload.userStats->address.len = 16;
+			} else {
+				sendmsg->payload.userStats->has_address = false;
+			}
 		}
 		/* BW */
 		sendmsg->payload.userStats->has_bandwidth = true;
